import path = require("path");
import fs = require('fs');
import StringUtil from "./StringUtil";
import {Config} from "./Config";
import readline = require('readline');

export default class FileUtil {
    /**
     * 递归删除文件夹及文件
     * @param dirPath 需要删除的文件夹路径
     */
    public static clearDir(dirPath: string) {
        if (fs.existsSync(dirPath)) {
            fs.readdirSync(dirPath).forEach((file) => {
                let curPath = path.join(dirPath, file);
                if (fs.statSync(curPath).isDirectory()) {
                    this.clearDir(curPath);
                } else {
                    fs.unlinkSync(curPath);
                }
            });
        }
    }

    /**
     * 检测一个文件夹路径是否存在，根据条件自动创建
     * @param dirPath
     */
    public static checkDir(dirPath: string, isAutoMake: boolean = false) {
        if (!fs.existsSync(dirPath)) {
            if (isAutoMake) {
                fs.mkdirSync(dirPath)
                return true;
            }
            return false;
        } else {
            return true;
        }
    }

    public static mkdirsSync(dirname) {
        if (fs.existsSync(dirname)) {
            return true;
        } else if (this.mkdirsSync(path.dirname(dirname))) {
            fs.mkdirSync(dirname);
            return true;
        }
        return false;
    }

    /**
     * 检测一个文件夹路径是否存在，如果没有就自动创建
     * @param dirPath
     */
    public static checkAndMakeDir(dirPath: string) {
        return this.mkdirsSync(dirPath)
    }

    /**
     * 获取一个目录下当前目录的全部文件名
     * @param dirPath
     * @param filter 扩展名筛选
     * @param isFilterTempFile 临时文件筛选
     */
    public static getFileList(dirPath: string, filter?: string[], isFilterTempFile: boolean = true): string[] {
        let ret: string[] = [];
        let files = fs.readdirSync(dirPath);
        for (let i = 0; i < files.length; i++) {
            let file = files[i];
            if (filter) {
                for (let j = 0; j < filter.length; j++) {
                    if (isFilterTempFile && file.indexOf('~$') != -1) {
                        break;
                    }
                    if (file.lastIndexOf(filter[j]) != -1) {
                        ret.push(file)
                        break;
                    }
                }
            } else {
                ret.push(file)
            }
        }
        return ret;
    }

    /**
     * 保存文本文件(流方式)
     * @param text
     * @param fileName
     * @param outPutDir
     */
    public static saveTextByStream(text: string, fileName: string, outPutDir: string = Config.outPutDir) {
        let savePath = path.join(outPutDir, fileName)
        let writerStream = fs.createWriteStream(savePath)
        writerStream.write(text, 'utf-8')
        writerStream.on('finish', () => {
            console.log(`${savePath} is finished`)
            writerStream.close();
        });
    }

    /**
     * 保存文本文件
     * @param text
     * @param fileName
     * @param outPutDir
     */
    public static saveText(text: string, fileName: string, outPutDir: string = Config.outPutDir) {
        let savePath = path.join(outPutDir, fileName)
        if (fs.existsSync(savePath)) {
            fs.unlinkSync(savePath)
        }
        fs.writeFileSync(savePath, text)
        console.log(`${savePath} is finished`)
    }


    // ----------------------------------------------
    /**
     * 保存为单独的ts格式文件
     * @param outPutData
     */
    public static saveTsAlone(outPutData) {
        let text = '/* eslint-disable */\r\n'
        let jsobj = outPutData.jsobj
        text += `export interface I${outPutData.sheetname} {`
        outPutData.headInfo.keys.forEach((key, index) => {
            let arrStr = ''
            for (let i = 1; i < key.arrayNum; i++) {
                arrStr += '[]'
            }
            if (key.outputclient) text += `\r\n\t${key.name}:${key.type}${arrStr}${index < outPutData.headInfo.keys.length - 1 ? ',' : ''} //${key.des.replace(/\s/g, '')}`
        })
        text += '\r\n}\r\n'
        text += `export let ${outPutData.sheetname}: { [key: string]: I${outPutData.sheetname} }=\r\n{`
        if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
            text = StringUtil.getTsFileString(jsobj, outPutData.sheetname, text, outPutData.headInfo)
        }
        text += '\r\n}\r\n'
        this.saveText(text, `${outPutData.sheetname}.ts`)
    }

    /**
     * 保存为单独的js格式文件
     * @param outPutData
     */
    public static saveJsAlone(outPutData) {
        let text = '/* eslint-disable */\r\n'
        let jsobj = outPutData.jsobj
        text += `module.exports =\r\n{`
        if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
            text = StringUtil.getTsFileString(jsobj, outPutData.sheetname, text, outPutData.headInfo)
        }
        text += '\r\n}\r\n'
        this.saveText(text, `${outPutData.sheetname}.js`)
    }

    /**
     * 保存为ts格式文件
     * @param outPutData
     */
    public static saveTs(outPutData) {
        let text = '/* eslint-disable */\r\n'
        for (let i = 0; i < outPutData.length; i++) {
            let jsobj = outPutData[i].jsobj

            if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
                text = StringUtil.getTsClassString(jsobj, outPutData[i].sheetname, text, outPutData[i].headInfo)
            }
        }
        text += 'export class Config\r\n{'
        for (let i = 0; i < outPutData.length; i++) {
            let jsobj = outPutData[i].jsobj
            if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
                text = StringUtil.getTsFileString(jsobj, outPutData[i].sheetname, text, outPutData[i].headInfo)
            }
        }
        text += '\r\n}\r\n'
        this.saveText(text, 'Config.ts')
    }

    public static saveTsJson(outPutData, isMin = false) {
        let text = '/* eslint-disable */\r\n'
        for (let i = 0; i < outPutData.length; i++) {
            let jsobj = outPutData[i].jsobj
            if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
                text = StringUtil.getTsClassString(jsobj, outPutData[i].sheetname, text, outPutData[i].headInfo)
            }
        }
        text += 'export class Config\r\n{'
        for (let i = 0; i < outPutData.length; i++) {
            let jsobj = outPutData[i].jsobj
            if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
                text = StringUtil.getTsFileHead(outPutData[i].sheetname, text);
                StringUtil.getTsFileString3(jsobj, outPutData[i].headInfo)
                let out = '';
                if (isMin) {
                    out = JSON.stringify(jsobj);
                } else {
                    out = JSON.stringify(jsobj, null, '\t');
                }
                // out = StringUtil.replaceAll(out, '\\"\\[', '[');
                // out = StringUtil.replaceAll(out, '\\]\\"', ']');
                out = out.replace(/\\\\/g, '\\');
                this.saveText(out, `${outPutData[i].sheetname}.json`, path.join(Config.outPutDir, Config.jsonOutputDir))
            }
        }
        text += '\r\n}\r\n'
        this.saveText(text, 'Config.ts', path.join(Config.outPutDir, Config.tsOutputDir))
    }

    /**
     * 保存为js格式文件
     * @param outPutData
     */
    public static saveJs(outPutData) {
        let text = '/* eslint-disable */\r\nmodule.exports = {\r\n'
        for (let i = 0; i < outPutData.length; i++) {
            let jsobj = outPutData[i].jsobj
            if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
                text = StringUtil.getJsFileString(jsobj, outPutData[i].sheetname, text, outPutData[i].headInfo)
            }
        }
        text += '}\r\n';
        this.saveText(text, 'Config.js')
    }

    public static saveJsJson(outPutData) {
        for (let i = 0; i < outPutData.length; i++) {
            let jsobj = outPutData[i].jsobj
            if (jsobj.length > 0 && JSON.stringify(jsobj[0]) != '{}') {
                this.saveText(JSON.stringify(jsobj, null, '\t'), `${outPutData[i].sheetname}.json`)
            }
        }
    }

    /**
     * 保存为csv格式文件
     * @param outPutData
     */
    public static saveCsv(outPutData) {
        for (let i = 0; i < outPutData.length; i++) {
            let csvobj = outPutData[i].csvobj
            if (csvobj.length > 0 && JSON.stringify(csvobj[0]) != '{}') {
                this.saveText(StringUtil.getCsvFileString(csvobj), `${outPutData[i].sheetname}.csv`, path.join(Config.outPutDir, 'csv'))
            }
        }
    }

    /**
     * 保存为hsv格式文件
     * @param outPutData
     */
    public static saveHsv(outPutData) {
        for (let i = 0; i < outPutData.length; i++) {
            let csvobj = outPutData[i].csvobj
            if (csvobj.length > 0 && JSON.stringify(csvobj[0]) != '{}') {
                this.saveText(StringUtil.getHsvFileString(csvobj), `${outPutData[i].sheetname}.hsv`, path.join(Config.outPutDir, 'hsv'))
            }
        }
    }

    /**
     * 将进度信息打印到同一行
     * @param text
     */
    public static printSameLine(text) {
        readline.clearLine(process.stdout, 0);         //删除光标所在行
        readline.cursorTo(process.stdout, 0)           //移动光标到行首
        process.stdout.write(text, 'utf-8');
    }

    public static printlog(val) {
        let index = Math.floor(val / Config.part)
        // console.log(val, index)
        if (index > Config.partIndex) {
            Config.partIndex = index;
            console.log(Config.part * index * 100 + '%')
        }
    }
}
